Index: src/netfpga/netfpga.c
===================================================================
--- src/netfpga/netfpga.c	(wersja 2080)
+++ src/netfpga/netfpga.c	(kopia robocza)
@@ -52,6 +52,8 @@
 #include "../../include/nf2.h"
 #include "../../include/netfpga_freebsd.h"
 
+#include "../libxbf/xbf.h"
+
 /*-------------------------------------------------------------------------
  * Global stuff.
  */
@@ -71,8 +73,9 @@
 	int fd;
 	int flag_v;
 	struct nf2err err;
-	
+
 	/* Convert to typedefs .. */
+	int (*init)(struct nf *nf2);
 	uint32_t (*reg_read32)(struct nf2 *nf2, unsigned long offset);
 	void (*reg_write32)(struct nf2 *nf2, unsigned long offset,
 	    uint32_t value);
@@ -149,6 +152,16 @@
 		return (e->msg_wo_debug);
 }
 
+/*-------------------------------------------------------------------------*/
+
+nf2_image_write();
+nf2_image_write_cpci();
+nf2_reg_read();
+nf2_reg_write();
+nf2_reg_list();
+
+nf2_reset();
+
 /*-------------------------------------------------------------------------
  * Context initialization and destruction.
  */
@@ -170,16 +183,21 @@
 	ep->fp = stderr;
 	ep->msg_wo_debug = NULL;
 
+#if 0
 	if (dev_path == NULL)
 		path = "/dev/nf0";
 	else
 		path = dev_path;
-
+	rer->sc = nf2_init_fbsd(
 	ret->fd = open(path, O_RDWR);
 	if (ret->fd == -1)
 		return (NF2_ERRP(ret, "Couldn't open '%s' in R/W mode", path));
+#endif
+	ret->init = nf2_init_fbsd;
+	ret->destroy = nf2_destroy_fbsd;
 	ret->reg_read32 = nf2_reg_read32_fbsd;
 	ret->reg_write32 = nf2_reg_write32_fbsd;
+	_nf2_start(sc);
 	return (ret);
 }
 
@@ -195,6 +213,18 @@
 	return (error);
 }
 
+static int
+_nf2_start(struct nf2 *nf2)
+{
+
+	ASSERT(nf2->init != NULL);
+
+	nf2->sc = nf2->init(sc);
+	if (nf2->sc == NULL)
+		return (NF2_ERR(nf2, "NetFPGA library initialization failed"));
+	
+}
+
 /*-------------------------------------------------------------------------
  * Context initialization and destruction.
  */
@@ -351,3 +381,30 @@
 		err(EXIT_FAILURE, nf2_errmsg_get(nf2));
 	exit(EXIT_SUCCESS);
 }
+
+struct netfpga {
+	int	which;
+};
+
+typedef netfpga_open_t;
+typedef netfpga_close_t;
+typedef netfpga_readreg_t;
+typedef netfpga_writereg_t;
+
+struct netfpga_handler nf2_linux = {
+	.version = 	NETFPGA_REV2,
+	.open =		nf2_linux_open,
+	.close =	nf2_linux_close,
+	.readreg =	nf2_linux_readreg,
+	.writereg =	nf2_linux_writereg,
+};
+NETFPGA_IO_MODULE(&nf2_linux);
+
+struct netfpga_handler nf2_freebsd = {
+	.version =	NETFPGA_REV2,
+	.open =		nf2_freebsd_open,
+	.close = 	nf2_freebsd_close,
+	.readreg =	nf2_freebsd_readreg,
+	.writereg =	nf2_linux_writereg,
+};
+NETFPGA_IO_MODULE(&nf_freebsd);
Index: src/libxbf/xbf.c
===================================================================
--- src/libxbf/xbf.c	(wersja 2080)
+++ src/libxbf/xbf.c	(kopia robocza)
@@ -29,37 +29,8 @@
 
 #define ASSERT	assert
 
-struct xbf {
-	void		*_xbf_mem;
-	size_t		 _xbf_memsize;
-#define _XBF_ERRMSG_LEN	1024
-	char		 _xbf_errmsg[_XBF_ERRMSG_LEN];
+#include "xbf.h"
 
-	const char	*xbf_ncdname;
-	const char	*xbf_partname;
-	const char	*xbf_time;
-	const char	*xbf_date;
-	uint32_t	 xbf_bitlen;
-	const char	*xbf_bitptr;
-};
-
-#define XBF_HDR_SIZE 72
-
-struct xbf *xbf_open(const char *fname);
-int xbf_close(struct xbf *xbf);
-const char *xbf_errmsg(struct xbf *xbf);
-struct xbf *xbf_err(struct xbf *xbf, const char *fmt, ...);
-int xbf_erri(struct xbf *xbf, const char *fmt, ...);
-uint32_t xbf_get_bitlen(struct xbf *xbf);
-const unsigned char *xbf_get_bitptr(struct xbf *xbf);
-const char *xbf_get_partname(struct xbf *xbf);
-const char *xbf_get_date(struct xbf *xbf);
-const char *xbf_get_time(struct xbf *xbf);
-const char *xbf_get_ncdname(struct xbf *xbf);
-void xbf_print_fp(FILE *fp, struct xbf *xp);
-void xbf_print(struct xbf *xbf);
-int xbf_opened(struct xbf *xbf);
-
 static int
 _xbf_setup(struct xbf *xbf)
 {
@@ -354,6 +325,8 @@
 	xbf_print_fp(stdout, xbf);
 }
 
+#ifdef XBF_TEST_PROG
+
 int
 main(int argc, char **argv)
 {
@@ -387,3 +360,4 @@
 
 	exit(EXIT_SUCCESS);
 }
+#endif /* XBF_TEST_PROG */
Index: src/netfpga_kmod/netfpga.c
===================================================================
--- src/netfpga_kmod/netfpga.c	(wersja 2080)
+++ src/netfpga_kmod/netfpga.c	(kopia robocza)
@@ -68,6 +68,9 @@
 	struct resource *irq;
 	void *intrhand;
 
+	bus_dma_tag_t	parent_tag;
+	bus_dma_tag_t	rx_tag;
+
 	void *mem_kva;
 	int need_reset;
 };
@@ -331,9 +334,6 @@
 
 	/* Reset all MACs */
 	nf2_mac_reset_all(sc);
-
-	/* Enable all MACs */
-	nf2_mac_enable_all(sc);
 }
 
 static void
@@ -341,6 +341,11 @@
 {
 
 	NF2_SOFTC_ASSERT(sc);
+
+	/* Enable all MACs */
+	nf2_mac_enable_all(sc);
+
+	/* Enable interrupts */
 	nf2_irq_enable(sc);
 }
 
@@ -379,6 +384,73 @@
 #endif
 }
 
+static int
+nf2_dma_alloc(struct nf2_softc *sc)
+{
+	int error = 0;
+
+	NF2_SOFTC_ASSERT(sc);
+
+	/*
+	 * Allocate the parent bus DMA tag appropriate for PCI.
+	 */
+	error = bus_dma_tag_create(
+	    bus_get_dma_tag(sc->dev),	/* parent */
+	    1,				/* alignment */
+	    0,				/* boundary */
+	    BUS_SPACE_MAXADDR_32BIT,	/* lowaddr */
+	    BUS_SPACE_MAXADDR,		/* highaddr */
+	    NULL,			/* filter */
+	    NULL,			/* filterarg */
+	    BUS_SPACE_MAXSIZE_32BIT,	/* maxsize */
+	    0,				/* nsegments */
+	    BUS_SPACE_MAXSIZE_32BIT,	/* maxsegsize */
+	    0,				/* flags */
+	    NULL,			/* lockfunc */
+	    NULL,			/* lockarg */
+	    &sc->parent_tag);
+	if (error) {
+		NF2_DEBUG("Couldn't create parent DMA tag: %d", error);
+		return (error);
+	}
+
+	/* Create DMA tag for Rx memory. */
+	error = bus_dma_tag_create(
+	    sc->parent_tag,		/* parent */
+	    1,				/* alignment */
+	    0,				/* boundary */
+	    BUS_SPACE_MAXADDR_32BIT,	/* lowaddr */
+	    BUS_SPACE_MAXADDR,		/* highaddr */
+	    NULL,			/* filter */
+	    NULL,			/* filterarg */
+	    BUS_SPACE_MAXSIZE_32BIT,	/* maxsize */
+	    0,				/* nsegments */
+	    BUS_SPACE_MAXSIZE_32BIT,	/* maxsegsize */
+	    0,				/* flags */
+	    NULL,			/* lockfunc */
+	    NULL,			/* lockarg */
+	    &sc->rx_tag);
+	if (error) {
+		NF2_DEBUG("Couldn't create DMA tag for RX mem: %d", error);
+		return (error);
+	}
+	return (error);
+}
+
+static int
+nf2_dma_free(struct nf2_softc *sc)
+{
+	int error = 0;
+
+	NF2_SOFTC_ASSERT(sc);
+	if (sc->parent_tag != NULL)
+		error = bus_dma_tag_destroy(sc->parent_tag);
+	if (sc->rx_tag != NULL)
+		error = bus_dma_tag_destroy(sc->rx_tag);
+	NF2_DEBUG("returning with error=%d", error);
+	return (error);
+}
+
 /*--------------------------------------------------------------------------*/
 
 static int
@@ -485,6 +557,7 @@
 	sc->cdev->si_drv1 = sc;
 
 	nf2_setup(sc);
+	nf2_dma_alloc(sc);
 
 	error = bus_setup_intr(dev, sc->irq, INTR_TYPE_NET | INTR_MPSAFE,
 	    NULL, nf2_intr, sc, &sc->intrhand);
@@ -522,6 +595,8 @@
 			NF2_DEBUG("Couldn't tear down interrupt");
 	}
 
+	nf2_dma_free(sc);
+
 	if (sc->cdev)
 		destroy_dev(sc->cdev);
 
