#ifndef _NETFPGA_H_
#define _NETFPGA_H_

#define ASSERT assert

extern int	netfpga_debug;
extern int	netfpga_tbd;
#define TBD()							\
	if (netfpga_tbd)					\
		printf("%s(%d): called\n", __func__, __LINE__)
#define DEBUG(...)						\
	if (netfpga_debug) printf(__VA_ARGS__)
#define DBGS(x)							\
	printf("%s = '%s'\n", #x, (x))

struct netfpga;
typedef void *netfpga_open_t(struct netfpga *nf);
typedef int netfpga_close_t(struct netfpga *nf, void *ctx);
typedef int netfpga_read_t(struct netfpga *nf, void *ctx, uint32_t reg,
    void *buf, size_t buf_len);
typedef int netfpga_write_t(struct netfpga *nf, void *ctx, uint32_t reg,
    void *buf, size_t buf_len);

struct netfpga_module {
	unsigned int		 nf_version;
	unsigned int		 nf_flags;
	const char		*nf_name;
	char			 __empty[64]; /* Future */

	netfpga_open_t		*nf_open;
	netfpga_close_t		*nf_close;
	netfpga_read_t		*nf_read;
	netfpga_write_t		*nf_write;
};

#define NETFPGA_MODULE(modname)						\
	const char str##modname[] __attribute__((section("netfpga"))) = \
	    #modname

#define NETFPGA_PATH_LIB	"."

struct netfpga {
	/* Debugging */
	unsigned		__nf_magic;
#define NETFPGA_MAGIC		0x3d34e37e
#define NETFPGA_INIT(n)		((n)->__nf_magic = NETFPGA_MAGIC)
#define NETFPGA_ASSERT(n)	(assert((n)->__nf_magic == NETFPGA_MAGIC))

	/* Error handling */
	FILE			*__nf_err_fp;
	char			 __nf_errmsg_src[512];
	char			 __nf_errmsg[1024];

	/* Private */
	void			*__nf_dll;
	struct netfpga_module	*__nf_mod;
	void			*__nf_mod_ctx;

	/* Public stuff */
	const char		*nf_iface;
	int			 nf_verbose;
};

static inline void
_netfpga_start(struct netfpga *nf)
{

	nf->__nf_err_fp = NULL;
	memset(nf->__nf_errmsg_src, 0, sizeof(nf->__nf_errmsg_src));
	memset(nf->__nf_errmsg, 0, sizeof(nf->__nf_errmsg));

	nf->__nf_dll = NULL;
	nf->__nf_mod = NULL;
	nf->__nf_mod_ctx = NULL;

	nf->nf_iface = NULL;
	nf->nf_verbose = 0;
}

int netfpga_start(struct netfpga *nf);
int netfpga_stop(struct netfpga *nf);
int netfpga_read(struct netfpga *nf, uint32_t reg, void *buf, size_t buf_len);
int netfpga_write(struct netfpga *nf, uint32_t reg, void *buf, size_t buf_len);
void netfpga_wr32(struct netfpga *nf, uint32_t reg, uint32_t value);
uint32_t netfpga_rd32(struct netfpga *nf, uint32_t reg);
void netfpga_image_devname(struct netfpga *nf, char **dev_name);
void _netfpga_err_fmt(struct netfpga *nf, const char *func, int lineno,
    const char *fmt, va_list va);
struct netfpga *_netfpga_err(struct netfpga *nf, const char *func, int lineno,
    const char *fmt, ...);
#define netfpga_err(nf, fmt, ...)					\
	(_netfpga_err(nf, __func__, __LINE__, fmt, ## __VA_ARGS__))
int _netfpga_erri(struct netfpga *nf, const char *func, int lineno,
    const char *fmt, ...);
#define netfpga_erri(nf, fmt, ...)					\
	(_netfpga_erri(nf, __func__, __LINE__, fmt, ## __VA_ARGS__))
int netfpga_has_error(struct netfpga *nf);
const char *netfpga_strerror(struct netfpga *nf);

#endif
